@@ -23,7 +23,7 @@ package MongoDB::Error;
 
 use version;
 
-our $VERSION = 'v2.0.3';
+our $VERSION = 'v2.2.0';
 
 use Moo;
 use Carp;
@@ -49,6 +49,7 @@ BEGIN {
         HOST_NOT_FOUND            => 7,
         UNKNOWN_ERROR             => 8,
         USER_NOT_FOUND            => 11,
+        ILLEGAL_OPERATION         => 20,
         NAMESPACE_NOT_FOUND       => 26,
         INDEX_NOT_FOUND           => 27,
         CURSOR_NOT_FOUND          => 43,
@@ -120,6 +121,13 @@ sub has_error_label {
     return grep { $_ eq $expected } @{ $self->error_labels };
 }
 
+sub add_error_label {
+    my ( $self, $label ) = @_;
+
+    return if $self->has_error_label( $label );
+    push @{ $self->error_labels }, $label;
+}
+
 sub throw {
   my ($inv) = shift;
 
@@ -144,23 +152,23 @@ sub _is_resumable { 1 }
 # logic.
 sub __is_retryable_error { 0 }
 
+my @retryable_codes = (
+    MongoDB::Error::HOST_NOT_FOUND(),
+    MongoDB::Error::HOST_UNREACHABLE(),
+    MongoDB::Error::NETWORK_TIMEOUT(),
+    MongoDB::Error::SHUTDOWN_IN_PROGRESS(),
+    MongoDB::Error::PRIMARY_STEPPED_DOWN(),
+    MongoDB::Error::SOCKET_EXCEPTION(),
+    MongoDB::Error::NOT_MASTER(),
+    MongoDB::Error::INTERRUPTED_AT_SHUTDOWN(),
+    MongoDB::Error::INTERRUPTED_DUE_TO_REPL_STATE_CHANGE(),
+    MongoDB::Error::NOT_MASTER_NO_SLAVE_OK(),
+    MongoDB::Error::NOT_MASTER_OR_SECONDARY(),
+);
+
 sub _check_is_retryable_code {
     my $code = $_[-1];
 
-    my @retryable_codes = (
-        MongoDB::Error::HOST_NOT_FOUND(),
-        MongoDB::Error::HOST_UNREACHABLE(),
-        MongoDB::Error::NETWORK_TIMEOUT(),
-        MongoDB::Error::SHUTDOWN_IN_PROGRESS(),
-        MongoDB::Error::PRIMARY_STEPPED_DOWN(),
-        MongoDB::Error::SOCKET_EXCEPTION(),
-        MongoDB::Error::NOT_MASTER(),
-        MongoDB::Error::INTERRUPTED_AT_SHUTDOWN(),
-        MongoDB::Error::INTERRUPTED_DUE_TO_REPL_STATE_CHANGE(),
-        MongoDB::Error::NOT_MASTER_NO_SLAVE_OK(),
-        MongoDB::Error::NOT_MASTER_OR_SECONDARY(),
-    );
-
     return 1 if grep { $code == $_ } @retryable_codes;
     return 0;
 }
@@ -196,6 +204,51 @@ sub _is_retryable {
     return $self->__is_retryable_error;
 }
 
+my @unknown_commit_codes = (
+    MongoDB::Error::EXCEEDED_TIME_LIMIT(),
+    MongoDB::Error::WRITE_CONCERN_ERROR(),
+);
+
+sub _check_is_unknown_commit_code {
+    my $code = $_[-1];
+
+    return 1 if grep { $code == $_ } @unknown_commit_codes;
+    return 0;
+}
+
+sub _is_unknown_commit_error {
+    my $self = shift;
+
+    return 1 if $self->isa("MongoDB::ConnectionError") || $self->isa("MongoDB::SelectionError");
+
+    return 1 if $self->_is_retryable;
+
+    if ( $self->$_can( 'result' ) ) {
+        return 1 if _check_is_unknown_commit_code( $self->result->last_code );
+    }
+
+    if ( $self->$_can( 'code' ) ) {
+        return 1 if _check_is_unknown_commit_code( $self->code );
+    }
+
+    return 0;
+}
+
+sub _is_transient_transaction_error {
+    my $self = shift;
+    return 1 if $self->isa("MongoDB::ConnectionError") || $self->isa("MongoDB::SelectionError");
+    return 0;
+}
+
+# Look for error code ILLEGAL_OPERATION and starts with "Transaction numbers"
+sub _is_storage_engine_not_retryable {
+    my $self = shift;
+    if ( $self->$_can( 'code' ) ) {
+        return 0 if $self->code != MongoDB::Error::ILLEGAL_OPERATION;
+    }
+    return index($self->message, "Transaction numbers", 0) == 0;
+}
+
 #--------------------------------------------------------------------------#
 # Subclasses with attributes included inline below
 #--------------------------------------------------------------------------#
@@ -416,7 +469,7 @@ MongoDB::Error - MongoDB Driver Error classes
 
 =head1 VERSION
 
-version v2.0.3
+version v2.2.0
 
 =head1 SYNOPSIS
 
