@@ -20,7 +20,7 @@ package MongoDB::Op::_ListIndexes;
 # Encapsulate index list operation; returns array ref of index documents
 
 use version;
-our $VERSION = 'v2.0.2';
+our $VERSION = 'v2.2.0';
 
 use Moo;
 
@@ -33,8 +33,10 @@ use Types::Standard qw(
   InstanceOf
 );
 use Tie::IxHash;
-use Try::Tiny;
 use Safe::Isa;
+use MongoDB::_Types qw(
+    ReadPreference
+);
 
 use namespace::clean;
 
@@ -44,6 +46,12 @@ has client => (
     isa      => InstanceOf ['MongoDB::MongoClient'],
 );
 
+has read_preference => (
+    is  => 'rw', # rw for Op::_Query which can be modified by Cursor
+    required => 1,
+    isa => ReadPreference,
+);
+
 with $_ for qw(
   MongoDB::Role::_PrivateConstructor
   MongoDB::Role::_CollectionOp
@@ -70,16 +78,18 @@ sub _command_list_indexes {
         query_flags => {},
         bson_codec  => $self->bson_codec,
         monitoring_callback => $self->monitoring_callback,
+        read_preference     => $self->read_preference,
+        session     => $self->session,
     );
 
-    my $res = try {
+    my $res = eval {
         $op->execute( $link, $topology );
-    }
-    catch {
-        if ( $_->$_isa("MongoDB::DatabaseError") ) {
-            return undef if $_->code == NAMESPACE_NOT_FOUND(); ## no critic: make $res undef
+    } or do {
+        my $error = $@ || "Unknown error";
+        unless ( $error->$_isa("MongoDB::DatabaseError") and $error->code == NAMESPACE_NOT_FOUND()) {
+            die $error;
         }
-        die $_;
+        undef;
     };
 
     return $res
@@ -100,7 +110,7 @@ sub _legacy_list_indexes {
         db_name             => $self->db_name,
         full_name           => $self->db_name . '.system.indexes',
         read_concern        => MongoDB::ReadConcern->new,
-        read_preference     => MongoDB::ReadPreference->new,
+        read_preference     => $self->read_preference || MongoDB::ReadPreference->new,
         monitoring_callback => $self->monitoring_callback,
     );
 
