@@ -20,7 +20,7 @@ package MongoDB::Role::_DatabaseOp;
 # and a BSON codec.  This is likely a "base role" for all operations.
 
 use version;
-our $VERSION = 'v2.0.3';
+our $VERSION = 'v2.2.0';
 
 use Moo::Role;
 
@@ -70,4 +70,11 @@ has retryable_write => (
     default => 0,
 );
 
+# set during retryable reads on supported operations
+has retryable_read => (
+    is      => 'rw',
+    isa     => Boolish,
+    default => 0,
+);
+
 1;
