@@ -20,7 +20,7 @@ package MongoDB::_Constants;
 # Common MongoDB driver constants
 
 use version;
-our $VERSION = 'v2.0.3';
+our $VERSION = 'v2.2.0';
 
 use Exporter 5.57 qw/import/;
 use Config;
@@ -37,13 +37,14 @@ BEGIN {
         MAX_BSON_OBJECT_SIZE         => 4_194_304,
         MAX_GRIDFS_BATCH_SIZE        => 16_777_216,                 # 16MiB
         MAX_BSON_WIRE_SIZE           => 16_793_600,                 # 16MiB + 16KiB
-        MAX_WIRE_VERSION             => 5,
+        MAX_WIRE_VERSION             => 8,
         MAX_WRITE_BATCH_SIZE         => 1000,
         MIN_HEARTBEAT_FREQUENCY_SEC  => .5,
         MIN_HEARTBEAT_FREQUENCY_USEC => 500_000,                    # 500ms, not configurable
         MIN_KEYED_DOC_LENGTH         => 8,
         MIN_SERVER_VERSION           => "2.4.0",
         MIN_WIRE_VERSION             => 0,
+        RESCAN_SRV_FREQUENCY_SEC      => $ENV{TEST_MONGO_RESCAN_SRV_FREQUENCY_SEC} || 60,
         NO_JOURNAL_RE                => qr/^journaling not enabled/,
         NO_REPLICATION_RE          => qr/^no replication has been enabled/,
         P_INT32                    => $] lt '5.010' ? 'l' : 'l<',
@@ -55,6 +56,14 @@ BEGIN {
         TXN_IN_PROGRESS             => 'in_progress',
         TXN_COMMITTED               => 'committed',
         TXN_ABORTED                 => 'aborted',
+        TXN_WTIMEOUT_RETRY_DEFAULT  => 10_000,  # 10 seconds
+        TXN_TRANSIENT_ERROR_MSG     => 'TransientTransactionError',
+        TXN_UNKNOWN_COMMIT_MSG      => 'UnknownTransactionCommitResult',
+        # From the Convenient API for Transactions spec, with_transaction must
+        # halt retries after 120 seconds.
+        # This limit is non-configurable and was chosen to be twice the 60 second
+        # default value of MongoDB's `transactionLifetimeLimitSeconds` parameter.
+        WITH_TXN_RETRY_TIME_LIMIT   => 120, # seconds
     };
 }
 
