@@ -20,7 +20,7 @@ package MongoDB::_Dispatcher;
 # allows avoiding circular references with the session pool class.
 
 use version;
-our $VERSION = 'v2.0.2';
+our $VERSION = 'v2.2.0';
 
 use Moo;
 use MongoDB::_Constants;
@@ -47,6 +47,12 @@ has retry_writes => (
     isa      => Boolish,
 );
 
+has retry_reads => (
+    is       => 'ro',
+    required => 1,
+    isa      => Boolish,
+);
+
 # Reset session state if we're outside an active transaction, otherwise set
 # that this transaction actually has operations
 sub _maybe_update_session_state {
@@ -65,10 +71,10 @@ sub send_direct_op {
 
     $self->_maybe_update_session_state( $op );
 
-    ( $link = $self->{topology}->get_specific_link($address) ), (
+    ( $link = $self->{topology}->get_specific_link( $address, $op ) ), (
         eval { ($result) = $op->execute($link); 1 } or do {
             my $err = length($@) ? $@ : "caught error, but it was lost in eval unwind";
-            if ( $err->$_isa("MongoDB::ConnectionError") ) {
+            if ( $err->$_isa("MongoDB::ConnectionError") || $err->$_isa("MongoDB::NetworkTimeout") ) {
                 $self->{topology}->mark_server_unknown( $link->server, $err );
             }
             elsif ( $err->$_isa("MongoDB::NotMasterError") ) {
@@ -82,6 +88,25 @@ sub send_direct_op {
       return $result;
 }
 
+sub _retrieve_link_for {
+    my ( $self, $op, $rw ) = @_;
+    my $topology = $self->{'topology'};
+    my $link;
+    if ( $op->session
+        && $op->session->_address # no point trying if theres no address....
+        && $op->session->_active_transaction # this is true during a transaction and on every commit
+        && $topology->_supports_mongos_pinning_transactions )
+    {
+        $link = $topology->get_specific_link( $op->session->_address, $op );
+    }
+    elsif ( $rw eq 'w' ) {
+        $link = $topology->get_writable_link( $op );
+    } else {
+        $link = $topology->get_readable_link( $op );
+    }
+    return $link;
+}
+
 # op dispatcher written in highly optimized style
 sub send_write_op {
     my ( $self, $op ) = @_;
@@ -89,11 +114,11 @@ sub send_write_op {
 
     $self->_maybe_update_session_state( $op );
 
-    ( $link = $self->{topology}->get_writable_link ), (
-        eval { ($result) = $self->_try_write_op_for_link( $link, $op ); 1 } or do {
+    ( $link = $self->_retrieve_link_for( $op, 'w' ) ), (
+        eval { ($result) = $self->_try_op_for_link( $link, $op ); 1 } or do {
             my $err = length($@) ? $@ : "caught error, but it was lost in eval unwind";
             WITH_ASSERTS ? ( confess $err ) : ( die $err );
-          }
+        }
       ),
       return $result;
 }
@@ -108,13 +133,13 @@ BEGIN {
 
 sub send_retryable_write_op {
     my ( $self, $op, $force ) = @_;
-
-    my $result;
-    my $link = $self->{topology}->get_writable_link;
+    my ( $link, $result ) = ( $self->_retrieve_link_for( $op, 'w' ) );
 
     $self->_maybe_update_session_state( $op );
 
-    # Need to force to do a retryable write on a Transaction Commit or Abort. $force is an override for retry_writes, but theres no point trying that if the link doesnt support it anyway.
+    # Need to force to do a retryable write on a Transaction Commit or Abort.
+    # $force is an override for retry_writes, but theres no point trying that
+    # if the link doesnt support it anyway.
     # This triggers on the following:
     # * $force is not set to 'force'
     #   (specifically for retrying writes in ending transaction operations)
@@ -126,7 +151,7 @@ sub send_retryable_write_op {
           && ! $op->session->_in_transaction_state( TXN_STARTING, TXN_IN_PROGRESS )
         )
     ) {
-        eval { ($result) = $self->_try_write_op_for_link( $link, $op ); 1 } or do {
+        eval { ($result) = $self->_try_op_for_link( $link, $op ); 1 } or do {
             my $err = length($@) ? $@ : "caught error, but it was lost in eval unwind";
             WITH_ASSERTS ? ( confess $err ) : ( die $err );
         };
@@ -142,17 +167,24 @@ sub send_retryable_write_op {
     $op->retryable_write( 1 );
 
     # attempt the op the first time
-    eval { ($result) = $self->_try_write_op_for_link( $link, $op ); 1 } or do {
+    eval { ($result) = $self->_try_op_for_link( $link, $op ); 1 } or do {
         my $err = length($@) ? $@ : "caught error, but it was lost in eval unwind";
 
+        if ( $err->$_call_if_can('_is_storage_engine_not_retryable') ) {
+            # Break encapsulation to rewrite the message, then rethrow.
+            $err->{message} = "This MongoDB deployment does not support retryable writes. Please add retryWrites=false to your connection string.";
+            die $err;
+        }
+
         # If the error is not retryable, then drop out
         unless ( $err->$_call_if_can('_is_retryable') ) {
             WITH_ASSERTS ? ( confess $err ) : ( die $err );
         }
 
         # Must check if error is retryable before getting the link, in case we
-        # get a 'no writable servers' error
-        my $retry_link = $self->{topology}->get_writable_link;
+        # get a 'no writable servers' error. In the case of a mongos retry,
+        # this will end up as the same server by design.
+        my $retry_link = $self->_retrieve_link_for( $op, 'w' );
 
         # Rare chance that the new link is not retryable
         unless ( $retry_link->supports_retryWrites ) {
@@ -160,15 +192,9 @@ sub send_retryable_write_op {
         }
 
         # Second attempt
-        eval { ($result) = $self->_try_write_op_for_link( $retry_link, $op ); 1 } or do {
+        eval { ($result) = $self->_try_op_for_link( $retry_link, $op ); 1 } or do {
             my $retry_err = length($@) ? $@ : "caught error, but it was lost in eval unwind";
-            # Only network or not_master errors should propogate on second attempt
-            if ( $retry_err->$_isa("MongoDB::ConnectionError")
-              || $retry_err->$_isa("MongoDB::NotMasterError") ) {
-                WITH_ASSERTS ? ( confess $retry_err ) : ( die $retry_err );
-            }
-            # die with original error otherwise
-            WITH_ASSERTS ? ( confess $err ) : ( die $err );
+            WITH_ASSERTS ? ( confess $retry_err ) : ( die $retry_err );
         };
     };
     # just in case this gets reused for some reason
@@ -176,17 +202,32 @@ sub send_retryable_write_op {
     return $result;
 }
 
+sub _is_primary_stepdown {
+    my ($self, $err, $link) = @_;
+    my $err_info = $err->{result}->{output};
+    my $err_code_name = '';
+    $err_code_name = $err_info->{'codeName'} if defined $err_info->{'codeName'};
+    my @other_errors = qw(ShutdownInProgress InterruptedAtShutdown);
+    my $not_master = (
+        $err->$_isa('MongoDB::NotMasterError')
+            || ( $err_info && $err_code_name eq 'NotMaster' )
+    ) && $link->max_wire_version < 8;
+    return (
+        $err_info && grep { $err_code_name eq $_ } @other_errors
+    ) || $not_master;
+}
+
 # op dispatcher written in highly optimized style
-sub _try_write_op_for_link {
+sub _try_op_for_link {
     my ( $self, $link, $op ) = @_;
     my $result;
     (
         eval { ($result) = $op->execute($link, $self->{topology}->type); 1 } or do {
             my $err = length($@) ? $@ : "caught error, but it was lost in eval unwind";
-            if ( $err->$_isa("MongoDB::ConnectionError") ) {
+            if ( $err->$_isa("MongoDB::ConnectionError") || $err->$_isa("MongoDB::NetworkTimeout") ) {
                 $self->{topology}->mark_server_unknown( $link->server, $err );
             }
-            elsif ( $err->$_isa("MongoDB::NotMasterError") ) {
+            elsif ( $self->_is_primary_stepdown($err, $link) ) {
                 $self->{topology}->mark_server_unknown( $link->server, $err );
                 $self->{topology}->mark_stale;
             }
@@ -197,6 +238,64 @@ sub _try_write_op_for_link {
     return $result;
 }
 
+sub send_retryable_read_op {
+    my ( $self, $op ) = @_;
+    my $result;
+
+    # Get transaction read preference if in a transaction.
+    if ( defined $op->session && $op->session->_active_transaction ) {
+        # Transactions may only read from primary in MongoDB 4.0, so get and
+        # check the read preference from the transaction settings as per
+        # transaction spec - see MongoDB::_TransactionOptions
+        $op->read_preference( $op->session->_get_transaction_read_preference );
+    }
+
+    my $link = $self->_retrieve_link_for( $op, 'r' );
+
+    $self->_maybe_update_session_state( $op );
+
+    if ( ! $link->supports_retryReads
+        || ! $self->retry_reads
+        || ( defined $op->session && $op->session->_in_transaction_state( TXN_STARTING, TXN_IN_PROGRESS ))
+    ) {
+        eval { ($result) = $self->_try_op_for_link( $link, $op ); 1 } or do {
+            my $err = length($@) ? $@ : "caught error, but it was lost in eval unwind";
+            WITH_ASSERTS ? ( confess $err ) : ( die $err );
+        };
+        return $result;
+    }
+
+    $op->session->_increment_transaction_id if $op->session;
+
+    $op->retryable_read( 1 );
+    # attempt the op the first time
+    eval { ($result) = $self->_try_op_for_link( $link, $op ); 1 } or do {
+        my $err = length($@) ? $@ : "caught error, but it was lost in eval unwind";
+
+        # If the error is not retryable, then drop out
+        unless ( $err->$_call_if_can('_is_retryable') ) {
+            WITH_ASSERTS ? ( confess $err ) : ( die $err );
+        }
+
+        my $retry_link = $self->_retrieve_link_for( $op, 'r' );
+
+        # Rare chance that the new link is not retryable
+        unless ( $retry_link->supports_retryReads ) {
+            WITH_ASSERTS ? ( confess $err ) : ( die $err );
+        }
+
+        # Second attempt
+        eval { ($result) = $self->_try_op_for_link( $retry_link, $op ); 1 } or do {
+            my $retry_err = length($@) ? $@ : "caught error, but it was lost in eval unwind";
+                WITH_ASSERTS ? ( confess $retry_err ) : ( die $retry_err );
+        };
+    };
+    # just in case this gets reused for some reason
+    $op->retryable_read( 0 );
+
+    return $result;
+}
+
 # op dispatcher written in highly optimized style
 sub send_read_op {
     my ( $self, $op ) = @_;
@@ -212,11 +311,11 @@ sub send_read_op {
 
     $self->_maybe_update_session_state( $op );
 
-    ( $link = $self->{topology}->get_readable_link( $op->read_preference ) ),
+    ( $link = $self->_retrieve_link_for( $op, 'r' ) ),
       ( $type = $self->{topology}->type ), (
         eval { ($result) = $op->execute( $link, $type ); 1 } or do {
             my $err = length($@) ? $@ : "caught error, but it was lost in eval unwind";
-            if ( $err->$_isa("MongoDB::ConnectionError") ) {
+            if ( $err->$_isa("MongoDB::ConnectionError") || $err->$_isa("MongoDB::NetworkTimeout") ) {
                 $self->{topology}->mark_server_unknown( $link->server, $err );
             }
             elsif ( $err->$_isa("MongoDB::NotMasterError") ) {
